using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DataSet_Xml
{
    using System.Xml;
    using System.IO;
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
              
        // Diese Methode konvertiert ein bergebenes DataSet in einen Xml-String:
        public string ConvertDataSetToXML(DataSet ds)
        {
            MemoryStream stream = null;
            XmlTextWriter writer = null;
            try
            {
                stream = new MemoryStream();
                // XmlTextWriter mit dem Memorystream initialiseren:
                writer = new  XmlTextWriter(stream, Encoding.Unicode);
                // DataSet in den MemoryStream schreiben:
                ds.WriteXml(writer, XmlWriteMode.WriteSchema);
                // Byte-Array als Puffer erstellen (Memorystream kann nur in Byte-Array einlesen):
                byte[] arr = stream.ToArray();
                UnicodeEncoding utf = new UnicodeEncoding();
                // Xml-String aus Byte-Array gewinnen und zurckgeben:
               return utf.GetString(arr).Trim();
            }
            catch
            {
                return String.Empty;
            }
            finally
            {
                if (writer != null) writer.Close();
            }
        }

        // Diese Methode konvertiert einen bergebenen Xml-String in ein DataSet:
        public DataSet ConvertXMLToDataSet(string xml)
        {
            StringReader reader = null;
            try
            {
                DataSet ds = new DataSet();
                reader = new StringReader(xml);
                // Xml-String in DataSet einlesen: 
                ds.ReadXml(reader);
                return ds;
            }
            catch
            {
                return null;
            }
            finally
            {
                if (reader != null) reader.Close();
            }
        }

        // Diese Methode konvertiert ein bergebenes untypisiertes in ein typisierte DataSet
        // (DataSet ds = (DataSet) ds1;   funktioniert, DataSet1 ds1 = (DataSet1) ds;  leider nicht):
        public DataSet1 ConvertUntypedToTypedDS(DataSet ds)
        {
            MemoryStream stream = null;
            try
            {
                stream = new MemoryStream();
                // DataSet inkl. Strukturinfo in den MemoryStream schreiben:
                ds.WriteXml(stream, XmlWriteMode.WriteSchema);
                // Position im Memorystream auf Anfang zurcksetzen:
                stream.Seek(0, SeekOrigin.Begin);
                // typisiertes DataSet erzeugen und Inhalt des Memorystreams einlesen:
                DataSet1 ds1 = new DataSet1();
                ds1.ReadXml(stream, XmlReadMode.InferSchema);             
                return ds1;
            }
            catch 
            {
               return null;
            }
            finally
            {
                if (stream != null) stream.Close();
            }
        }

        // Methode erzeugt ein untypisiertes Test-DataSet mit zwei DataTables ("Personen" und "Bestellungen")
        // und einer Relation und fgt jeder Tabelle zwei Datenstze hinzu
        private DataSet getTestDS()
        {
            // ----------------- Tabelle "Kunden" --------------------------------------

            DataTable dt1 = new DataTable("Kunden");
            // Primrschlssel:
            DataColumn col1 = dt1.Columns.Add("Nr", Type.GetType("System.Int32"));
            col1.AllowDBNull = false;
            col1.Unique = true;
            col1.AutoIncrement = true;
            col1.AutoIncrementStep = 1;

            dt1.Columns.Add("Vorname", typeof(System.String));
            dt1.Columns.Add("Nachname", typeof(System.String));
            dt1.Columns.Add("Geburtstag", typeof(System.DateTime));

            // Datenstze hinzufgen:
            DataRow rw11 = dt1.NewRow();
            rw11["Vorname"] = "Klaus"; rw11["Nachname"] = "Mller"; rw11["Geburtstag"] = Convert.ToDateTime("3.4.1975");
            DataRow rw12 = dt1.NewRow();
            rw12["Vorname"] = "Tobalt"; rw12["Nachname"] = "Tonne"; rw12["Geburtstag"] = Convert.ToDateTime("5.8.1984");
            dt1.Rows.Add(rw11);
            dt1.Rows.Add(rw12);
            
            // --------------------- Tabelle "Bestellungen" ----------------------------

            DataTable dt2 = new DataTable("Bestellungen");
            // Primrschlssel:
            DataColumn col2 = dt2.Columns.Add("Nr", Type.GetType("System.Int32"));
            col2.AllowDBNull = false;
            col2.Unique = true;
            col2.AutoIncrement = true;
            col2.AutoIncrementStep = 1;

            dt2.Columns.Add("Datum", typeof(System.DateTime));
            dt2.Columns.Add("Betrag", typeof(System.Decimal));
            dt2.Columns.Add("KuNr", typeof(System.Int32));        // Fremdschlssel
            dt2.Columns.Add("Bemerkung", typeof(System.String));

            // Datenstze hinzufgen:
            DataRow rw21 = dt2.NewRow();
            rw21["Datum"] = Convert.ToDateTime("20.2.2008"); rw21["Betrag"] = Convert.ToDecimal("256,50");
            rw21["KuNr"] = 0; rw21["Bemerkung"] = "per Nachname";
            dt2.Rows.Add(rw21);
            DataRow rw22 = dt2.NewRow();
            rw22["Datum"] = Convert.ToDateTime("8.3.2008"); rw22["Betrag"] = Convert.ToDecimal("12,95");
            rw22["KuNr"] = 0;  rw22["Bemerkung"] = ("per Scheck");
            dt2.Rows.Add(rw22);

            // ------------------------------------------------------------------------

            // DataSet zusammenbauen (mit 1 : n Relation zwischen Kunden und Bestellungen):
            DataSet ds = new DataSet();
            ds.Tables.Add(dt1);
            ds.Tables.Add(dt2);
            ds.Relations.Add("Kunde_Bestellungen", ds.Tables["Kunden"].Columns["Nr"],
                                                    ds.Tables["Bestellungen"].Columns["KuNr"]);
            return ds;
        }

        // DataSet => Xml-String:
        private void button1_Click(object sender, EventArgs e)
        {
           DataSet ds = getTestDS();
           textBox1.Text = ConvertDataSetToXML(ds);
           button2.Enabled = true;
        }

        // Xml-String => DataSet:
        private void button2_Click(object sender, EventArgs e)
        {
            DataSet ds = ConvertXMLToDataSet(textBox1.Text);
            dataGrid1.DataSource = null;
            dataGrid1.DataSource = ds;
            MessageBox.Show(ds.Tables["Kunden"].Rows[0]["Vorname"].ToString());
        }
      
    }
}