using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WindowsApplication5
{
    public partial class Form1 : Form
    {
        private DataTable dtA;        // Datencache fr die Tabelle "Ausgaben" 

        private DataTable dtMWSt;    // Hilfstabelle

        public Form1()
        {
            InitializeComponent();

            // Hilfstabelle erzeugen:
            dtMWSt = createMWStTbl();
        }

        // Hilfstabelle zur Anzeige der gltigen MWSt-Stze erzeugen:
        private DataTable createMWStTbl()
        {
            DataTable dt = new DataTable("MWSt");
            dt.Columns.Add("Nr", typeof(System.Byte));
            dt.Columns.Add("Betrag", typeof(System.String));

            string[] ma = new string[3] { "keine", "7%", "19%" };
            for (int i = 1; i <= 3; i++)         // 3 Zeilen hinzufgen
            {
                dt.Rows.Add(dt.NewRow());              //  neue leere Zeile
                dt.Rows[i - 1][1] = ma[i - 1];         // "Betrag" eintragen
            }
            dt.Rows[0][0] = 0;    // "Nr" eintragen (0, 7, 19)
            dt.Rows[1][0] = 7;
            dt.Rows[2][0] = 19;
            return dt;
        }

        // Anzeigen:
        private void button1_Click(object sender, EventArgs e)
        {
            BindingSource bs = new BindingSource();
            try
            {
                dtA = CData.getAusgaben(textBox1.Text, textBox2.Text);
                if (dtA == null)
                {
                    dataGridView1.DataSource = null;
                    toolStripStatusLabel1.Text = "Zugriff nicht mglich!";
                }
                else
                {
                    bs.DataSource = dtA;
                    dataGridView1.Columns.Clear();
                    dataGridView1.DataSource = bs;
 
                    // ---------------------------------------------------------------

                    #region formatColumns
                    
                    dataGridView1.Columns.Remove("Nr");
                    DataGridViewTextBoxColumn tbc0 = new DataGridViewTextBoxColumn();
                    tbc0.DataPropertyName = "Nr";
                    tbc0.HeaderText = "Nr";
                    tbc0.Width = 30;
                    tbc0.DisplayIndex = 0;
                    dataGridView1.Columns.Add(tbc0);
                    
                    // ---------------------------------------------------------------------

                    // Datumsspalte wird nicht formatiert, behlt Standardwerte
                    
                    // ---------------------------------------------------------------------

                    dataGridView1.Columns.Remove("Netto");
                    DataGridViewTextBoxColumn tbc1 = new DataGridViewTextBoxColumn();
                    tbc1.DataPropertyName = "Netto";
                    tbc1.HeaderText = "Netto";
                    tbc1.Width = 80;                         // Breite einstellen
                    tbc1.DefaultCellStyle.Format = "c";
                    tbc1.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                    tbc1.DefaultCellStyle.Font = new Font(dataGridView1.Font, FontStyle.Bold);
                    tbc1.DefaultCellStyle.BackColor = Color.LightYellow;
                    tbc1.DisplayIndex = 2;
                    dataGridView1.Columns.Add(tbc1);

                    // -------------------------------------------------------------------

                    dataGridView1.Columns.Remove("MWSt");
                    DataGridViewComboBoxColumn cbc0 = new DataGridViewComboBoxColumn();
                    cbc0.DataSource = dtMWSt;                // Detailtabelle 
                    cbc0.DataPropertyName = "MWSt";          // Fremdschlssel  aus Primrtabelle
                    cbc0.ValueMember = "Nr";                 // Primrschlssel aus Detailtabelle
                    cbc0.DisplayMember = "Betrag";           // Detailanzeige aus Detailtabelle
                    cbc0.HeaderText = "MWSt";
                    cbc0.DisplayIndex = 3;
                    cbc0.Width = 60;
                    cbc0.DisplayStyle = DataGridViewComboBoxDisplayStyle.Nothing;
                    dataGridView1.Columns.Add(cbc0);
                   
                    //-------------------------------------------------------------
                    
                    // Spalte "Bemerkungen" wird nicht formatiert, behlt Standardwerte

                    // ---------------------------------------------------------------------
                    #endregion

                    toolStripStatusLabel1.Text = bs.Count.ToString() + " Einnahmen zum Editieren von der Datenbank geladen!";
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Fehler beim Laden der DataTable!");
            }
        }
               

        // Speichern:
        private void button2_Click(object sender, EventArgs e)
        {
            if (dtA != null)
            {
                try
                {
                    // Nur die genderten Datenstze werden zurckgeschrieben:
                        DataTable dt1 = dtA.GetChanges();       // nderungen ermitteln
                        if (dt1 != null)
                        {
                            CData.setAusgaben(ref dt1);         // Datenbank-Update ausfhren                       
                            dtA.Merge(dt1);          // Zusammenfhren beider DataTables
                            dtA.AcceptChanges();
                            toolStripStatusLabel1.Text = "Alle nderungen wurden von der Datenbank bernommen!";
                        }
                        else
                        {
                            toolStripStatusLabel1.Text = "Keine nderungen zum Speichern!";
                        }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message.ToString(), "Fehler beim Speichern der DataTable!");
                    dtA.RejectChanges();
                }
            }
            else
            {
                toolStripStatusLabel1.Text = "Speichern nicht mglich!";
            }
        }

        // Beenden:
        private void button3_Click(object sender, EventArgs e)
        {
            this.Close();
        }

    }
}