﻿using System;
using System.Data;
using System.Data.Objects;
using System.Data.Objects.DataClasses;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Configuration;
using System.Collections;
using System.Collections.Specialized;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml.Linq;
using System.Web.DynamicData;

namespace Dynamic_Data
{
    public partial class ManyToMany_EditField : System.Web.DynamicData.FieldTemplateUserControl
    {
        public void Page_Load(object sender, EventArgs e)
        {
            // Für Aktualisierungsereignis der Datenquelle registrieren
            EntityDataSource ds = (EntityDataSource)this.FindDataSourceControl();

            // Diese Feldvorlage wird sowohl für das Bearbeiten als auch für das Einfügen verwendet
            ds.Updating += new EventHandler<EntityDataSourceChangingEventArgs>(DataSource_UpdatingOrInserting);
            ds.Inserting += new EventHandler<EntityDataSourceChangingEventArgs>(DataSource_UpdatingOrInserting);
        }

        void DataSource_UpdatingOrInserting(object sender, EntityDataSourceChangingEventArgs e)
        {
            MetaTable childTable = ChildrenColumn.ChildTable;

            // Kommentare setzen zu Illustrationszwecken Mitarbeiter-/Gebietsangaben voraus, aber der Code ist generisch

            // Auflistung der Gebiete für diesen Mitarbeiter abrufen
            RelatedEnd entityCollection = (RelatedEnd)Column.EntityTypeProperty.GetValue(e.Entity, null);

            // Im Bearbeitungsmodus sicherstellen, dass sie geladen ist (im Einfügemodus nicht sinnvoll)
            if (Mode == DataBoundControlMode.Edit && !entityCollection.IsLoaded)
            {
                entityCollection.Load();
            }

            // IList daraus abrufen (Liste der Gebiete für aktuellen Mitarbeiter)
            // PRÜFEN: EntityCollection sollte direkt verwendet werden, aber EF hat keinen
            // generischen Typ dafür. Dieser wird in vnext hinzugefügt.
            IList entityList = ((IListSource)entityCollection).GetList();

            // Alle Gebiete durchlaufen (nicht nur die für diesen Mitarbeiter)
            foreach (object childEntity in childTable.GetQuery(e.Context))
            {

                // Überprüfen, ob der Mitarbeiter derzeit dieses Gebiet hat
                bool isCurrentlyInList = entityList.Contains(childEntity);

                // Kontrollkästchen für dieses Gebiet suchen, um neuen Zustand festzustellen
                string pkString = childTable.GetPrimaryKeyString(childEntity);
                ListItem listItem = CheckBoxList1.Items.FindByValue(pkString);
                if (listItem == null)
                    continue;

                // Wenn sich der Zustand geändert hat, entsprechende Änderung (Hinzufügen/Entfernen) vornehmen
                if (listItem.Selected)
                {
                    if (!isCurrentlyInList)
                        entityList.Add(childEntity);
                }
                else
                {
                    if (isCurrentlyInList)
                        entityList.Remove(childEntity);
                }
            }
        }

        protected void CheckBoxList1_DataBound(object sender, EventArgs e)
        {
            MetaTable childTable = ChildrenColumn.ChildTable;

            // Kommentare setzen zu Illustrationszwecken Mitarbeiter-/Gebietsangaben voraus, aber der Code ist generisch

            IList entityList = null;
            ObjectContext objectContext = null;

            if (Mode == DataBoundControlMode.Edit)
            {
                object entity;
                ICustomTypeDescriptor rowDescriptor = Row as ICustomTypeDescriptor;
                if (rowDescriptor != null)
                {
                    // Echte Entity aus Wrapper abrufen
                    entity = rowDescriptor.GetPropertyOwner(null);
                }
                else
                {
                    entity = Row;
                }

                // Auflistung der Gebiete für diesen Mitarbeiter abrufen und sicherstellen, dass sie geladen ist
                RelatedEnd entityCollection = Column.EntityTypeProperty.GetValue(entity, null) as RelatedEnd;
                if (entityCollection == null)
                {
                    throw new InvalidOperationException(String.Format("Die ManyToMany-Vorlage unterstützt den Auflistungstyp der Spalte '{0}' in Tabelle '{1}' nicht.", Column.Name, Table.Name));
                }
                if (!entityCollection.IsLoaded)
                {
                    entityCollection.Load();
                }

                // IList daraus abrufen (Liste der Gebiete für aktuellen Mitarbeiter)
                // PRÜFEN: EntityCollection sollte direkt verwendet werden, aber EF hat keinen
                // generischen Typ dafür. Dieser wird in vnext hinzugefügt.
                entityList = ((IListSource)entityCollection).GetList();

                // Aktuellen ObjectContext abrufen
                // PRÜFEN: Diese Vorgehensweise ist nicht sehr elegant. Bessere Alternative suchen.
                ObjectQuery objectQuery = (ObjectQuery)entityCollection.GetType().GetMethod(
                    "CreateSourceQuery").Invoke(entityCollection, null);
                objectContext = objectQuery.Context;
            }

            // Alle Gebiete durchlaufen (nicht nur die für diesen Mitarbeiter)
            foreach (object childEntity in childTable.GetQuery(objectContext))
            {
                MetaTable actualTable = MetaTable.GetTable(childEntity.GetType());
                // Kontrollkästchen dafür erstellen
                ListItem listItem = new ListItem(
                    actualTable.GetDisplayString(childEntity),
                    actualTable.GetPrimaryKeyString(childEntity));

                // Aktivieren, falls aktueller Mitarbeiter dieses Gebiet hat
                if (Mode == DataBoundControlMode.Edit)
                {
                    listItem.Selected = entityList.Contains(childEntity);
                }

                CheckBoxList1.Items.Add(listItem);
            }
        }

        public override Control DataControl
        {
            get
            {
                return CheckBoxList1;
            }
        }

    }
}
