using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Data.SqlClient;
using System.Security.Permissions;


namespace QueryNotifications
{
    public partial class Form1 : Form
    {
        private const string SQLSTR = "SELECT CustomerID, CompanyName, ContactName, ContactTitle FROM dbo.Customers ORDER BY CompanyName";

        private SqlConnectionStringBuilder csb = null;
        private SqlConnection conn = null;
        private SqlCommand cmd = null;
        private DataSet ds = null;

        private int count = 0;
        private const string msg = "{0} nderungen sind aufgetreten.";

        // Um SqlDependency nutzen zu knnen, muss Anwendung ber SqlClientPermission verfgen:
        private bool canRequestNotifications()
        {
            try
            {
                SqlClientPermission perm = new SqlClientPermission(PermissionState.Unrestricted);
                perm.Demand();
                return true;
            }
            catch
            {
                return false;
            }
        }

        // Anfangsaktivitten:
        public Form1()
        {
           InitializeComponent();

            // Clientberechtigung prfen:
           button1.Enabled = canRequestNotifications();
            // ConnectionString zusammenbauen:
           csb = new SqlConnectionStringBuilder();
           csb.DataSource = ".\\SQLEXPRESS";
           csb.IntegratedSecurity = true;
           csb.InitialCatalog = "northwind";
        }

        private void getData()
        {
            // Kehraus:
            ds.Clear();

            // Sicherstellen, dass Command-Objekt nicht bereits mit einem Notification-Objekt verknpft ist:
            cmd.Notification = null;

            // SqlDependency-Objekt erzeugen und mit Command-Objekt verbinden:
            SqlDependency dep = new SqlDependency(cmd);
            // Eventhandler anmelden:
            dep.OnChange += new OnChangeEventHandler(dep_OnChange);

            // Daten laden und anzeigen:
            SqlDataAdapter da = new SqlDataAdapter(cmd);
            da.Fill(ds, "Customers");
            dataGridView1.DataSource = ds;
            dataGridView1.DataMember = "Customers";
        }

        // mit Sql Server verbinden:
        private void button1_Click(object sender, EventArgs e)
        {
            count = 0;
            toolStripStatusLabel1.Text = String.Format(msg, count);

            // existierende Dependency-Connection anhalten und neu starten:
            SqlDependency.Stop(csb.ConnectionString);
            SqlDependency.Start(csb.ConnectionString);

            if (conn == null)
            {
                conn = new SqlConnection(csb.ConnectionString);
            }

            if (cmd == null)
            {
                cmd = new SqlCommand(SQLSTR, conn);
            }

            if (ds == null)
            {
                ds = new DataSet();
            }
            getData();
        }
                    
        // Der OnChange-Eventhandler: 
        private void dep_OnChange(object sender, SqlNotificationEventArgs e)
        {
            // Da das OnChange-Event nicht im Thread der Benutzerschnittstelle (UI-Thread),
            // sondern im Arbeitsthread (aus dem Threadpool) auftritt, ist ein Aktualisieren
            // der Benutzerschnittstelle zunchst nicht mglich. 
            // Der folgende Code fhrt deshalb die notwendigen Aktionen aus, 
            // um vom Arbeitstread  zum UI-Thread umzuschalten und die Ereignisbehandlung erneut zu registrieren.

            ISynchronizeInvoke isi = (ISynchronizeInvoke) this;

            // Wenn InvokeRequired true liefert, wird Code noch im Arbeitsthread ausgefhrt:
            if (isi.InvokeRequired)
            {
                // Delegate fr Thread-Schalter erzeugen:
                OnChangeEventHandler tmpDeleg = new OnChangeEventHandler(dep_OnChange);
                object[] args = { sender, e };

                // Daten-Marshalling vom Arbeitsthread zum UI-Thread:
                isi.BeginInvoke(tmpDeleg, args);
                return;
            }

            // ab hier wird Code wieder im UI-Thread ausgefhrt:

            // Notification-Handler wird nicht mehr bentigt:
            SqlDependency dep = (SqlDependency)sender;
            dep.OnChange -= dep_OnChange;

            ++count;
            toolStripStatusLabel1.Text = String.Format(msg, count);

            // Zusatzinfos anzeigen:
            listBox1.Items.Clear();
            listBox1.Items.Add("Type: " + e.Type.ToString());
            listBox1.Items.Add("Source: " + e.Source.ToString());
            listBox1.Items.Add("Info: " + e.Info.ToString());

            // genderte Daten laden und anzeigen:
            getData();
        }
        
        // Schlussaktivitten:
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            SqlDependency.Stop(csb.ConnectionString);
            if (conn != null)
            {
                conn.Close();
            }
        }
    }
}