/****
Dieses SQL-Skript wurde über das Dialogfeld "Datensynchronisierung konfigurieren"
generiert. Dieses Skript enthält Anweisungen, mit denen die 
Spalten zur Änderungsnachverfolgung, die Tabelle für gelöschte Elemente und die Trigger
für die Serverdatenbank erstellt werden. Diese Datenbankobjekte sind für Synchronisierungsdienste
erforderlich, damit die Daten zwischen Client- und Serverdatenbanken
erfolgreich synchronisiert werden können. Weitere Informationen finden Sie im Hilfethema
"How to: Configure a Database Server for Synchronization".
****/


IF @@TRANCOUNT > 0
set ANSI_NULLS ON 
set QUOTED_IDENTIFIER ON 

GO
BEGIN TRANSACTION;


IF @@TRANCOUNT > 0
ALTER TABLE [Order Details] 
ADD [LastEditDate] DateTime NULL CONSTRAINT [DF_Order Details_LastEditDate] DEFAULT (GETUTCDATE()) WITH VALUES
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
ALTER TABLE [Order Details] 
ADD [CreationDate] DateTime NULL CONSTRAINT [DF_Order Details_CreationDate] DEFAULT (GETUTCDATE()) WITH VALUES
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[Order Details_Tombstone]')) 
BEGIN 
CREATE TABLE [Order Details_Tombstone]( 
    [OrderID] Int NOT NULL,
    [ProductID] Int NOT NULL,
    [DeletionDate] DateTime NULL
) ON [PRIMARY] 
END 

GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
ALTER TABLE [Order Details_Tombstone] ADD CONSTRAINT [PKDEL_Order Details_Tombstone_OrderIDProductID]
   PRIMARY KEY CLUSTERED
    ([OrderID], [ProductID])
    ON [PRIMARY]
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Order Details_DeletionTrigger') AND type = 'TR') 
   DROP TRIGGER [Order Details_DeletionTrigger] 

GO
CREATE TRIGGER [Order Details_DeletionTrigger] 
    ON [Order Details] 
    AFTER DELETE 
AS 
SET NOCOUNT ON 
UPDATE [Order Details_Tombstone] 
    SET [DeletionDate] = GETUTCDATE() 
    FROM deleted 
    WHERE deleted.[OrderID] = [Order Details_Tombstone].[OrderID] 
    AND deleted.[ProductID] = [Order Details_Tombstone].[ProductID] 
IF @@ROWCOUNT = 0 
BEGIN 
    INSERT INTO [Order Details_Tombstone] 
    ([OrderID], [ProductID], DeletionDate)
    SELECT [OrderID], [ProductID], GETUTCDATE()
    FROM deleted 
END 

GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Order Details_UpdateTrigger') AND type = 'TR') 
   DROP TRIGGER [Order Details_UpdateTrigger] 

GO
CREATE TRIGGER [Order Details_UpdateTrigger] 
    ON [Order Details] 
    AFTER UPDATE 
AS 
BEGIN 
    SET NOCOUNT ON 
    UPDATE [Order Details] 
    SET [LastEditDate] = GETUTCDATE() 
    FROM inserted 
    WHERE inserted.[OrderID] = [Order Details].[OrderID] 
    AND inserted.[ProductID] = [Order Details].[ProductID] 
END;
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Order Details_InsertTrigger') AND type = 'TR') 
   DROP TRIGGER [Order Details_InsertTrigger] 

GO
CREATE TRIGGER [Order Details_InsertTrigger] 
    ON [Order Details] 
    AFTER INSERT 
AS 
BEGIN 
    SET NOCOUNT ON 
    UPDATE [Order Details] 
    SET [CreationDate] = GETUTCDATE() 
    FROM inserted 
    WHERE inserted.[OrderID] = [Order Details].[OrderID] 
    AND inserted.[ProductID] = [Order Details].[ProductID] 
END;
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;
COMMIT TRANSACTION;
