/****
Dieses SQL-Skript wurde über das Dialogfeld "Datensynchronisierung konfigurieren"
generiert. Dieses Skript enthält Anweisungen, mit denen die 
Spalten zur Änderungsnachverfolgung, die Tabelle für gelöschte Elemente und die Trigger
für die Serverdatenbank erstellt werden. Diese Datenbankobjekte sind für Synchronisierungsdienste
erforderlich, damit die Daten zwischen Client- und Serverdatenbanken
erfolgreich synchronisiert werden können. Weitere Informationen finden Sie im Hilfethema
"How to: Configure a Database Server for Synchronization".
****/


IF @@TRANCOUNT > 0
set ANSI_NULLS ON 
set QUOTED_IDENTIFIER ON 

GO
BEGIN TRANSACTION;


IF @@TRANCOUNT > 0
ALTER TABLE [Products] 
ADD [LastEditDate] DateTime NULL CONSTRAINT [DF_Products_LastEditDate] DEFAULT (GETUTCDATE()) WITH VALUES
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
ALTER TABLE [Products] 
ADD [CreationDate] DateTime NULL CONSTRAINT [DF_Products_CreationDate] DEFAULT (GETUTCDATE()) WITH VALUES
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[Products_Tombstone]')) 
BEGIN 
CREATE TABLE [Products_Tombstone]( 
    [ProductID] Int NOT NULL,
    [DeletionDate] DateTime NULL
) ON [PRIMARY] 
END 

GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
ALTER TABLE [Products_Tombstone] ADD CONSTRAINT [PKDEL_Products_Tombstone_ProductID]
   PRIMARY KEY CLUSTERED
    ([ProductID])
    ON [PRIMARY]
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Products_DeletionTrigger') AND type = 'TR') 
   DROP TRIGGER [Products_DeletionTrigger] 

GO
CREATE TRIGGER [Products_DeletionTrigger] 
    ON [Products] 
    AFTER DELETE 
AS 
SET NOCOUNT ON 
UPDATE [Products_Tombstone] 
    SET [DeletionDate] = GETUTCDATE() 
    FROM deleted 
    WHERE deleted.[ProductID] = [Products_Tombstone].[ProductID] 
IF @@ROWCOUNT = 0 
BEGIN 
    INSERT INTO [Products_Tombstone] 
    ([ProductID], DeletionDate)
    SELECT [ProductID], GETUTCDATE()
    FROM deleted 
END 

GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Products_UpdateTrigger') AND type = 'TR') 
   DROP TRIGGER [Products_UpdateTrigger] 

GO
CREATE TRIGGER [Products_UpdateTrigger] 
    ON [Products] 
    AFTER UPDATE 
AS 
BEGIN 
    SET NOCOUNT ON 
    UPDATE [Products] 
    SET [LastEditDate] = GETUTCDATE() 
    FROM inserted 
    WHERE inserted.[ProductID] = [Products].[ProductID] 
END;
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;


IF @@TRANCOUNT > 0
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'Products_InsertTrigger') AND type = 'TR') 
   DROP TRIGGER [Products_InsertTrigger] 

GO
CREATE TRIGGER [Products_InsertTrigger] 
    ON [Products] 
    AFTER INSERT 
AS 
BEGIN 
    SET NOCOUNT ON 
    UPDATE [Products] 
    SET [CreationDate] = GETUTCDATE() 
    FROM inserted 
    WHERE inserted.[ProductID] = [Products].[ProductID] 
END;
GO
IF @@ERROR <> 0 
     ROLLBACK TRANSACTION;
COMMIT TRANSACTION;
