﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Data.SQLite;
using System.IO;

namespace Start
{
    public partial class Form1 : Form
    {
        SQLiteConnection conn;
        DataSet ds;
        SQLiteDataAdapter da;

        public Form1()
        {
            InitializeComponent();
        }


        [SQLiteFunction(Name = "Levenshtein", Arguments = 2, FuncType = FunctionType.Scalar)]
        class myLevenshtein : SQLiteFunction
        {
            public override object Invoke(object[] args)
            {
                string s = args[0].ToString();
                string t = args[1].ToString();

                int n = s.Length;
                int m = t.Length;
                int[,] d = new int[n + 1, m + 1];

                if (n == 0) return m;
                if (m == 0) return n;

                for (int i = 0; i <= n; d[i, 0] = i++)
                {}

                for (int j = 0; j <= m; d[0, j] = j++)
                {}

                for (int i = 1; i <= n; i++)
                {
                    for (int j = 1; j <= m; j++)
                    {
                        int cost = (t[j - 1] == s[i - 1]) ? 0 : 1;
                        d[i, j] = Math.Min(
                                           Math.Min(d[i - 1, j] + 1, d[i, j - 1] + 1),
                                           d[i - 1, j - 1] + cost);
                    }
                }
                return d[n, m];
            }
        }

        [SQLiteFunction(Name = "MaxLength", Arguments = -1, FuncType = FunctionType.Aggregate)]
        class MyMaxLength : SQLiteFunction
        {
            public override void Step(object[] args, int nStep, ref object contextData)
            {
                string s =  args[0].ToString();
                if (contextData == null)
                    contextData = 0;
                else
                    contextData = Math.Max((int)contextData, s.Length);
            }

            public override object Final(object contextData)
            {
                return contextData;
            }
        }



        private void button1_Click(object sender, EventArgs e)
        {
            conn = new SQLiteConnection("Data Source=" + AppDomain.CurrentDomain.BaseDirectory + "northwindef.db");
            conn.Open();
            da = new SQLiteDataAdapter("SELECT lastname, Levenshtein(lastname, 'Bucha') As Distanz FROM employees", conn);
            ds = new DataSet();
            da.Fill(ds, "Mitarbeiter");
            dataGridView1.DataSource = ds;
            dataGridView1.DataMember = "Mitarbeiter";
            conn.Close();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            conn = new SQLiteConnection("Data Source=" + AppDomain.CurrentDomain.BaseDirectory + "northwindef.db");
            conn.Open();
            SQLiteCommand cmd = conn.CreateCommand();
            cmd.CommandText = "SELECT MaxLength(lastname) FROM employees";

            int i =  Convert.ToInt32(cmd.ExecuteScalar());
            MessageBox.Show("Ergebnis = " + i.ToString());    
        }

    }
}
