﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Data.Objects;
using System.Data.Common;

using System.Data.EntityClient;
using System.Data.Metadata.Edm;
using System.Data.Objects.DataClasses;
using System.Configuration;
using System.Data.SqlClient;

namespace Kapitelbeispiel
{
    public partial class Form1 : Form
    {
        VereinDBEntities ctx;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ctx = new VereinDBEntities();
            ctx.SavingChanges += new EventHandler(ctx_SavingChanges);
            ctx.ObjectStateManager.ObjectStateManagerChanged += new CollectionChangeEventHandler(ObjectStateManager_ObjectStateManagerChanged);
        }

        void ctx_SavingChanges(object sender, EventArgs e)
        {
            var entries = ctx.ObjectStateManager.GetObjectStateEntries(EntityState.Added|EntityState.Modified|EntityState.Deleted);
            foreach (var entry in entries)
            {
                if (entry.EntityKey.EntitySetName == "Mitglieder")
                {
                    var en = (Mitglieder)entry.Entity;
                    if (en.EntityState == EntityState.Added)  // NEU
                    {
                        if (en.Vorname == "Peter")
                            throw new InvalidOperationException("Peter wollen wir nicht ...");
                        // ...                     
                    }
                    else if (en.EntityState == EntityState.Modified)  // Geändert
                    {
                    }
                    else if (en.EntityState == EntityState.Deleted)   // Gelöscht
                    {
                    }

                }
            }
        }

        void ObjectStateManager_ObjectStateManagerChanged(object sender, CollectionChangeEventArgs e)
        {
            listBox1.Items.Insert(0, e.Action.ToString() + ": " + e.Element.ToString());
        }

        private void button2_Click(object sender, EventArgs e)
        {
            dataGridView1.DataSource = ctx.Mitglieder;
            // lästigerweise werden auch die Navigation-Properties angezeigt, also weg damit
            try
            {
                dataGridView1.Columns.Remove("Vereinsvorsitze");
                dataGridView1.Columns.Remove("Vereinsmitgliedschaften");
            }
            catch
            { }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            try
            {
                ctx.SaveChanges();
            }
            catch (OptimisticConcurrencyException)
            {
                ctx.Refresh(System.Data.Objects.RefreshMode.StoreWins, ctx.Mitglieder);
                ctx.SaveChanges();
            } 

        }

        private void button4_Click(object sender, EventArgs e)
        {
            Mitglieder m = Mitglieder.CreateMitglieder(0, "Hans", "Glück");
            // ärgerlicher Weise wird auch die ID abgefragt, obwohl diese schreibgeschützt ist 
            ctx.AddToMitglieder(m);
            // alternative Schreibweise
            ctx.AddToMitglieder(Mitglieder.CreateMitglieder(0, "Paul", "Waldner"));
            ctx.AddToMitglieder(Mitglieder.CreateMitglieder(0, "Werner", "Wiesengrün"));
            // weitere Alternativen
            ctx.Mitglieder.AddObject(Mitglieder.CreateMitglieder(0, "Alexander", "Lehmann"));
            ctx.AddObject("Mitglieder", Mitglieder.CreateMitglieder(0, "Erwin", "Wunderlich"));
            
            Mitglieder m2 = ctx.CreateObject<Mitglieder>();
            m2.Nachname = "Wilhelm";
            m2.Vorname = "Otto";
        }

        private void button5_Click(object sender, EventArgs e)
        {
            Vereine v = new Vereine();
            v.Name = "Taubenzüchter";
            v.Ort = "Bad Taubhausen";
            v.PLZ = "12345";
            v.Strasse = "Bienengasse 333";
            v.Mitgliedsliste.Add(ctx.Mitglieder.Where(m => m.Nachname == "Glück").First());
            // hinzufügen des Vorsitzenden bzw. ablegen in der Liste
            ctx.Mitglieder.Where(m => m.Nachname == "Glück").First().VereinsVorsitze.Add(v);
        }

        private void button6_Click(object sender, EventArgs e)
        {
            // Erst der Verein
            Vereine v = new Vereine();
            v.Name = "C#-Freunde";
            v.Ort = "Frankfurt";
            v.PLZ = "54321";
            v.Strasse = "Programiererweg 5";
            // Dann die Mitglieder
            v.Mitgliedsliste.Add(Mitglieder.CreateMitglieder(0, "Thomas", "Gewinnus"));
            v.Mitgliedsliste.Add(Mitglieder.CreateMitglieder(0, "Walter", "Doberenz"));
            // den Vorsitzenden dürfen wir nicht vergessen zuzuweisen
            v.Vorsitzender = v.Mitgliedsliste.Where(m => m.Nachname == "Gewinnus").First();
            // und das Ganze noch in die Vereinsliste einfügen
            ctx.AddToVereine(v);
        }

        private void button7_Click(object sender, EventArgs e)
        {
            Vereine ver = ctx.Vereine.First(v => v.Name.Contains("C#"));
            MessageBox.Show(ver.Name + ": " + ver.EntityState.ToString());
            ctx.Dispose();
            ver.Name = "C#-Freunde 2008 e.V.";
            MessageBox.Show(ver.Name + ": " + ver.EntityState.ToString());
            
            // hier könnte serialisiert etc. werden 
            // irgendwann später ...

            ctx = new VereinDBEntities();
            // Originalobjekt in Context laden !!!
            //Vereine old = ctx.Vereine.First(v => v.Name.Contains("C#"));
            //ctx.ApplyPropertyChanges("Vereine",ver);

            // Alternativ mit
            EntityKey ek = ctx.CreateEntityKey("Vereine", ver);
            Object old;
            if (ctx.TryGetObjectByKey(ek, out old))
                ctx.ApplyPropertyChanges("Vereine", ver);

            ctx.SaveChanges();
        }

        private void button8_Click(object sender, EventArgs e)
        {
            int id = Int32.Parse(dataGridView1.CurrentRow.Cells[0].Value.ToString());  // so eine komplizierter Zugriff !!!
            var ver = ctx.Mitglieder.Include("VereinsMitgliedschaften").FirstOrDefault(m => m.Id == id).VereinsMitgliedschaften;
            // alternativ
            // var ver = ctx.Mitglieder.FirstOrDefault(m => m.Id == id).VereinsMitgliedschaften;
            //ver.Load(); // ab hier sind erst relevante Daten enthalten

            dataGridView2.DataSource = ver;
        }

        private void button9_Click(object sender, EventArgs e)
        {
            var ml = ctx.Vereine.Include("Mitgliedsliste").Where(v => v.Name.Contains("C#")).FirstOrDefault().Mitgliedsliste;
            dataGridView2.DataSource = ml;
        }

        private void button10_Click(object sender, EventArgs e)
        {
            var verein = ctx.Vereine.Include("Vorsitzender").Where(v => v.Name.Contains("C#")).FirstOrDefault();
            MessageBox.Show(verein.Name);
            // alternativ zu Include verein.VorsitzenderReference.Load();
            MessageBox.Show(verein.Vorsitzender.Vorname + " " + verein.Vorsitzender.Nachname);
        }

        private void button11_Click(object sender, EventArgs e)
        {

        }

        private void button11_Click_1(object sender, EventArgs e)
        {
            listBox1.Items.Clear();
            foreach (var en in ctx.ObjectStateManager.GetObjectStateEntries(EntityState.Modified))
            {
                listBox1.Items.Add("Änderung für: " + en.EntityKey.EntityContainerName + "."
                                                    + en.EntityKey.EntitySetName.ToString() + "." +
                                                    en.EntityKey.EntityKeyValues.First().Key + " = " + en.EntityKey.EntityKeyValues.First().Value);
                for (int i = 0; i < en.CurrentValues.FieldCount; i++)
                    listBox1.Items.Add(en.CurrentValues.DataRecordInfo.FieldMetadata[i].FieldType.ToString() + ": " + en.OriginalValues[i] + " -> " + en.CurrentValues[i]);
                listBox1.Items.Add("");
            }
            foreach (var en in ctx.ObjectStateManager.GetObjectStateEntries(EntityState.Deleted))
            {
                listBox1.Items.Add("Gelöscht: " + en.EntityKey.EntityContainerName + "."
                                               + en.EntityKey.EntitySetName.ToString() + "." +
                                               en.EntityKey.EntityKeyValues.First().Key + " = " + en.EntityKey.EntityKeyValues.First().Value);
                listBox1.Items.Add("");
            }
            foreach (var en in ctx.ObjectStateManager.GetObjectStateEntries(EntityState.Added))
            {
                listBox1.Items.Add("Hinzugefügt: " + en.EntityKey.EntityContainerName + "."
                                                   + en.EntityKey.EntitySetName.ToString());
                for (int i = 0; i < en.CurrentValues.FieldCount; i++)
                    listBox1.Items.Add(en.CurrentValues.DataRecordInfo.FieldMetadata[i].FieldType.ToString() + ": " + en.CurrentValues[i] );

                listBox1.Items.Add("");
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        private void button12_Click(object sender, EventArgs e)
        {
            using (EntityConnection conn = new EntityConnection("Name=VereinDBEntities"))
            {
                conn.Open();
                EntityCommand cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT VALUE m FROM VereinDBEntities.Mitglieder AS m ORDER BY m.Nachname";
                DbDataReader dr = cmd.ExecuteReader(CommandBehavior.SequentialAccess);
                while (dr.Read())
                    listBox1.Items.Add(dr["Nachname"].ToString());
                dr.Close();
            }

        }

        private void button13_Click(object sender, EventArgs e)
        {
            VereinDBEntities ctx = new VereinDBEntities();
            var query = ctx.CreateQuery<Mitglieder>("SELECT VALUE m FROM VereinDBEntities.Mitglieder AS m ORDER BY m.Nachname"); //.Include("Vereinsmitgliedschaften");
            dataGridView1.DataSource = query;

        }


        private void button14_Click(object sender, EventArgs e)
        {
            VereinDBEntities ctx = new VereinDBEntities();
            var query = from m in ctx.Mitglieder
                        select new {Name = ctx.TestFunktion(m.Vorname, m.Nachname), Vorname = m.Vorname};
            dataGridView1.DataSource = query;

        }

        private void button15_Click(object sender, EventArgs e)
        {
            ProbeModell db = new ProbeModell();
            db.CreateDatabase();
            var query = db.KundenMenge;
            dataGridView1.DataSource = query;
        }

        private void button16_Click(object sender, EventArgs e)
        {
            using (EntityConnection conn = new EntityConnection("Name=VereinDBEntities"))
            {
                conn.Open();
                EntityCommand cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT VALUE COUNT(m.id) FROM VereinDBEntities.Mitglieder AS m";
                MessageBox.Show("Mitglieder: " + cmd.ExecuteScalar().ToString());
                MessageBox.Show(cmd.ToTraceString());
            }

        }

        private void button17_Click(object sender, EventArgs e)
        {
            using (EntityConnection conn = new EntityConnection("Name=VereinDBEntities"))
            {
                conn.Open();
                EntityCommand cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT VALUE m FROM VereinDBEntities.Mitglieder AS m WHERE m.Nachname LIKE @Nachname";
                cmd.Parameters.AddWithValue("Nachname", "Ge%");
                DbDataReader dr = cmd.ExecuteReader(CommandBehavior.SequentialAccess);
                while (dr.Read())
                    listBox1.Items.Add(dr["Nachname"].ToString());
                dr.Close();
            }

        }

        private void button18_Click(object sender, EventArgs e)
        {
            VereinDBEntities ctx = new VereinDBEntities();
            ctx.ContextOptions.LazyLoadingEnabled = false;
            var vereine = ctx.Vereine;
            foreach (Vereine v in vereine)
            {
                MessageBox.Show(v.Name);
                v.VorsitzenderReference.Load();
                MessageBox.Show(v.Vorsitzender.Nachname);
            }
         
        }

        private void button19_Click(object sender, EventArgs e)
        {
            VereinDBEntities ctx = new VereinDBEntities();
            ctx.ContextOptions.LazyLoadingEnabled = false;

            var mitglied = (Mitglieder)ctx.GetObjectByKey(new EntityKey("VereinDBEntities.Mitglieder", "Id", 8));
            MessageBox.Show(mitglied.Nachname);
        }

 

        private void button20_Click(object sender, EventArgs e)
        {
            VereinDBEntities ctx = new VereinDBEntities();
            ctx.ExecuteStoreCommand("INSERT INTO mitglieder(Vorname, Nachname) VALUES ('Gunther', 'Weinberger');");
        }

        private void button21_Click(object sender, EventArgs e)
        {
            VereinDBEntities ctx = new VereinDBEntities();
            var q = ctx.ExecuteStoreQuery<Mitglieder>("SELECT * FROM mitglieder WHERE nachname LIKE 'G%'", "Mitglieder", MergeOption.PreserveChanges);

            var m = q.FirstOrDefault();
            m.Nachname = "Gewinnuuuus";
            ctx.SaveChanges();          
        }

        private void button22_Click(object sender, EventArgs e)
        {
            string conns = ConfigurationManager.ConnectionStrings["VereinDBEntities"].ConnectionString;
            EntityConnectionStringBuilder csb = new EntityConnectionStringBuilder(conns);
            MessageBox.Show(csb.ProviderConnectionString);

            SqlConnectionStringBuilder sb =new SqlConnectionStringBuilder(csb.ProviderConnectionString);
            sb.AttachDBFilename = @"|DataDirectory|\db\VereinDB.mdf";
            csb.ProviderConnectionString = sb.ToString();

            MessageBox.Show(csb.ProviderConnectionString);
            
            VereinDBEntities db = new VereinDBEntities(csb.ToString());
            dataGridView1.DataSource = db.Mitglieder;
        }

        private void button23_Click(object sender, EventArgs e)
        {

        }

        private void button22_Click_1(object sender, EventArgs e)
        {
        }



    }
}
